<?php

namespace EventPortal\Tests\Unit\Approval\Models;

use EventPortal\Domain\Approval\Models\Approval as ApprovalModel;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class ApprovalTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function event_approval_is_audited_when_updated(): void
    {
        config(['audit.console' => true]); // This needs to be before create, or no update audit is created

        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create();

        $approval->update(['status' => 'declined']);

        $this->assertCount(2, $approval->fresh()->audits); // Created and updated

        $updatedAudit = $approval->fresh()->audits->last();

        $this->assertArraySubset(['status' => 'awaiting-approval'], $updatedAudit->old_values);

        $this->assertArraySubset(['status' => 'declined'], $updatedAudit->new_values);
    }

    /** @test */
    public function event_status_match_returns_true(): void
    {
        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create();

        $this->assertTrue($approval->doesCurrentStatusMatchNewStatus('awaiting-approval'));
    }

    /** @test */
    public function event_status_not_match_returns_false(): void
    {
        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create();

        $this->assertFalse($approval->doesCurrentStatusMatchNewStatus('line-manager-approved'));
    }

    /** @test */
    public function will_return_correct_title_for_awaiting_approval_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create();

        $this->assertEquals('Awaiting', $approval->getStatusTitleAttribute());
    }

    /** @test */
    public function will_return_correct_title_for_line_manager_approved_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('line-manager-approved')->create();

        $this->assertEquals('Line Manager Approved', $approval->getStatusTitleAttribute());
    }

    /** @test */
    public function will_return_correct_title_for_approved_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('approved')->create();

        $this->assertEquals('Approved', $approval->getStatusTitleAttribute());
    }

    /** @test */
    public function will_return_correct_title_for_declined_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('declined')->create();

        $this->assertEquals('Declined', $approval->getStatusTitleAttribute());
    }

    /** @test */
    public function will_return_correct_description_for_awaiting_approval_status(): void
    {
        $event = factory(Event::class)->create();

        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create(['event_id' => $event->id]);

        $this->assertEquals(
            'Your event needs resubmitting for approval due to update that you have made to 
                    your event.',
            $approval->getStatusDescriptionAttribute($event->name)
        );
    }

    /** @test */
    public function will_return_correct_description_for_line_manager_approved_status(): void
    {
        $event = factory(Event::class)->create();

        $approval = factory(ApprovalModel::class)->state('line-manager-approved')->create(['event_id' => $event->id]);

        $this->assertEquals(
            'Your event is awaiting approval from the compliance manager.',
            $approval->getStatusDescriptionAttribute($event->name)
        );
    }

    /** @test */
    public function will_return_correct_description_for_approved_status(): void
    {
        $event = factory(Event::class)->create();

        $approval = factory(ApprovalModel::class)->state('approved')
            ->create(['event_id' => $event->id]);

        $this->assertEquals(
            'Your event has been approved! You can now proceed with ' . $event->name . '.',
            $approval->getStatusDescriptionAttribute($event->name)
        );
    }

    /** @test */
    public function will_return_correct_description_for_declined_status(): void
    {
        $event = factory(Event::class)->create();

        $approval = factory(ApprovalModel::class)->state('declined')->create(['event_id' => $event->id]);

        $this->assertEquals(
            'Your event has been declined. Read the notes provided to see where your event 
                    needs improvement.',
            $approval->getStatusDescriptionAttribute($event->name)
        );
    }

    /** @test */
    public function will_return_correct_alert_for_awaiting_approval_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('awaiting-approval')->create();

        $this->assertEquals('warning', $approval->getStatusAlertAttribute());
    }

    /** @test */
    public function will_return_correct_alert_for_line_manager_approved_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('line-manager-approved')->create();

        $this->assertEquals('warning', $approval->getStatusAlertAttribute());
    }

    /** @test */
    public function will_return_correct_alert_for_approved_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('approved')->create();

        $this->assertEquals('success', $approval->getStatusAlertAttribute());
    }

    /** @test */
    public function will_return_correct_alert_for_declined_status(): void
    {
        $approval = factory(ApprovalModel::class)->state('declined')->create();

        $this->assertEquals('error', $approval->getStatusAlertAttribute());
    }
}
