<?php

namespace EventPortal\Tests\Unit\Models;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;

class TeamTest extends TestCase
{
    use DatabaseMigrations, InteractsWithTeams, SeedRolesAndPermissions;

    /** @test **/
    public function has_events()
    {
        $team = factory(Team::class)->create();
        $eventInTeam = factory(Event::class)->create();
        $eventInTeam->teams()->sync($team->id);
        $eventNotInTeam = factory(Event::class)->create();
        $events = $team->events;

        $this->assertCount(1, $events);
        $this->assertContainsModel($eventInTeam, $events);
        $this->assertNotContainsModel($eventNotInTeam, $events);
    }

    /** @test **/
    public function has_users()
    {
        $team = factory(Team::class)->create();
        $userInTeam = factory(Admin::class)->create();
        $userInTeam->teams()->sync($team->id);
        $userNotInTeam = factory(Admin::class)->create();
        $users = $team->admins;

        $this->assertCount(1, $users);
        $this->assertContainsModel($userInTeam, $users);
        $this->assertNotContainsModel($userNotInTeam, $users);
    }

    /** @test **/
    public function can_determine_whether_a_model_belongs_to_a_team()
    {
        $team = factory(Team::class)->create();
        $eventInTeam = factory(Event::class)->create();
        $eventInTeam->teams()->sync($team->id);
        $eventNotInTeam = factory(Event::class)->create();

        $this->assertTrue($eventInTeam->belongsToTeam($team));
        $this->assertFalse($eventNotInTeam->belongsToTeam($team));
    }

    /** @test **/
    public function can_determine_whether_two_models_are_in_the_same_team()
    {
        $team = factory(Team::class)->create();
        $event1InTeam = factory(Event::class)->create();
        $event2InTeam = factory(Event::class)->create();
        $team->events()->sync([
            $event1InTeam->id,
            $event2InTeam->id
        ]);
        $eventNotInTeam = factory(Event::class)->create();

        $this->assertTrue($event1InTeam->belongsToSameTeamAs($event2InTeam));
        $this->assertTrue($event2InTeam->belongsToSameTeamAs($event1InTeam));
        $this->assertFalse($eventNotInTeam->belongsToSameTeamAs($event1InTeam));
        $this->assertFalse($eventNotInTeam->belongsToSameTeamAs($event2InTeam));
    }

    /** @test */
    public function filters_out_teams_for_admins_who_are_in_a_team(): void
    {
        $team = factory(Team::class)->create();
        $adminTeam = factory(Team::class)->create();
        $admin = factory(Admin::class)->state('admin')->create();

        $this->addToSameTeam([$admin], $adminTeam);

        $this->actingAsAdmin($admin);
        $result = Team::forAdminInTeam()->get();

        $this->assertCount(1, $result);
        $this->assertNotContainsModel($team, $result);
    }

    /** @test */
    public function does_not_filter_out_teams_for_super_admins_who_are_in_a_team(): void
    {
        $team = factory(Team::class)->create();
        $superAdminTeam = factory(Team::class)->create();
        $superAdmin = factory(Admin::class)->state('super-admin')->create();

        $this->addToSameTeam([$superAdmin], $superAdminTeam);

        $this->actingAsSuperAdmin($superAdmin);
        $result = Team::forAdminInTeam()->get();

        $this->assertCount(2, $result);
        $this->assertContainsModel($superAdminTeam, $result);
        $this->assertContainsModel($team, $result);
    }
}
