<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Util\Statistics\Facades\Statistics;
use EventPortal\Domain\Util\Statistics\Stat;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Str;
use Spatie\Snapshots\MatchesSnapshots;

class StatisticsTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function statistics_can_be_generated()
    {
        $statistics = Statistics::make('events.test')
            ->add(function (): array {
                return [
                    Stat::make('Stats 1', 34),
                    Stat::make('Stats 2', 11),
                ];
            })
            ->get();

        $this->assertMatchesJsonSnapshot($statistics);
    }

    /** @test */
    public function get_correct_statistics_by_name()
    {
        Statistics::make('one')
            ->add(function (): array {
                return [
                    Stat::make('Stats 1', 1),
                ];
            });

        Statistics::make('two')
            ->add(function (): array {
                return [
                    Stat::make('Stats 2', 2),
                ];
            });

        $statsOne = Statistics::make('one')->get();
        $statsTwo = Statistics::make('two')->get();

        $this->assertMatchesJsonSnapshot($statsOne);
        $this->assertMatchesJsonSnapshot($statsTwo);
    }

    /** @test **/
    public function can_add_stats_for_multiple_models()
    {
        $event = factory(Event::class)->create([
            'name' => 'Event name',
            'slug' => 'event-name',
        ]);

        $venue = factory(Venue::class)->create([
            'name' => 'Venue name that is long',
        ]);

        $stats = Statistics::make('stats.test')
            ->context($event, $venue)
            ->add(function (Event $event): array {
                return [
                    Stat::make('Stat 1', Str::length($event->name)),
                ];
            })
            ->add(function (Venue $venue): array {
                return [
                    Stat::make('Stat 2', Str::length($venue->name)),
                ];
            })
            ->get();

        $this->assertMatchesJsonSnapshot($stats);
    }

    /** @test **/
    public function can_add_stats_without_passing_context()
    {
        $event = factory(Event::class)->create([
            'name' => 'Event name',
            'slug' => 'event-name',
        ]);

        factory(Venue::class)->create([
            'name' => 'Venue name that is something else',
        ]);

        $stats = Statistics::make('stats.test')
            ->context($event)
            ->add(function (Event $event): array {
                return [
                    Stat::make('Stat 1', Str::length($event->name)),
                ];
            })
            ->add(function (): array {
                return [
                    Stat::make('Stat 2', Str::length(Venue::first()->name)),
                ];
            })
            ->get();

        $this->assertMatchesJsonSnapshot($stats);
    }
}
